#========================================================================================
# package      DE_EPAGES::Shop::t::SecondaryDomains
# base         DE_EPAGES::Database::API::Test
# state        private
#========================================================================================
package DE_EPAGES::Shop::t::SecondaryDomains;
use base DE_EPAGES::Database::API::Test;

use strict;
use Test::More tests => 18;
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::WebInterface::API::FakeRequest;
use DE_EPAGES::WebService::API::Client;
use HTTP::Status qw ( HTTP_MOVED_PERMANENTLY );
use SOAP::Transport::HTTP;

my $ROOTPATH = '/Test';
my $PRIMARYDOMAIN = 'localhost';
my $SECONDARYDOMAINS = [
    'testshop.de',
    'testshop.com',
    'test.shop.es',
];
my $REQUEST = '/TestCategory';

sub init {
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    ImportObjects('SecondaryDomains.xml', $ROOTPATH);
    return;
}

sub cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return;
}

sub _TestUmlautSecondary {
    ImportObjects('SecondaryDomains2.xml', $ROOTPATH);
    _Check301Redirect('http://xn--testshp-f1a.de/TestCategory', "http://$PRIMARYDOMAIN/TestCategory");
}

sub _Request {
    my ($URI) = @_;

    my $UserAgent = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $Response = $UserAgent->get($URI);
    return ($Response->code, $Response->content);
}

sub _Check301Redirect {
    my ($URI, $ExpectedTarget) = @_;
    my ($HttpCode, $Content) = _Request($URI);

    is $HttpCode, HTTP_MOVED_PERMANENTLY, "_Check301Redirect: status code ($URI)";
    is $Content, 'Moved to: ' . $ExpectedTarget, "_Check301Redirect: content ($URI)";
}

sub _TestRedirect {
    # secondary domains should redirect to primary domain (HTTP 301)
    foreach my $Protocol (qw(http https)) {
        my $Primary = "http://$PRIMARYDOMAIN$REQUEST";
        foreach my $SecondaryDomain (@$SECONDARYDOMAINS) {
            my $Secondary = "$Protocol://$SecondaryDomain$REQUEST";
            _Check301Redirect($Secondary, $Primary);
        }
    }
}

sub _TestShopService {
    my ($DomainName) = @_;
    my $Login = 'wsadmin';
    my $Password = 'wsadmin';

    my $FactoryService = DE_EPAGES::WebService::API::Client
        ->uri('urn://epages.de/WebService/FactoryService/2004/08')
        ->proxy("http://%2fUsers%2f$Login:$Password@" . "$DomainName/epages/Store.soap");

    my $ShopService = DE_EPAGES::WebService::API::Client
        ->uri('urn://epages.de/WebService/ShopService/2004/08')
        ->proxy("http://%2fUsers%2f$Login:$Password@" . "$DomainName/epages/Store.soap");

    my $ShopID = $FactoryService->objectByPath( '/Test/TestShop' )->result;
    my $aPreviousDomains = $ShopService->getSecondaryDomains($ShopID)->result;

    my $TempDomain = 'aaaaa.invalid';
    $ShopService->setSecondaryDomains($ShopID, [$TempDomain, $DomainName]);

    my $aSecondaryDomains = $ShopService->getSecondaryDomains($ShopID)->result;
    unless(scalar(@$aSecondaryDomains) == 2 && $aSecondaryDomains->[0] eq $TempDomain) {
        print("$TempDomain not inserted\n");
        return 0;
    }

    $ShopService->setSecondaryDomains($ShopID, $aPreviousDomains);
    $aSecondaryDomains = $ShopService->getSecondaryDomains($ShopID)->result;
    foreach my $Secondary (@$SECONDARYDOMAINS) {
        unless(ElementOf($Secondary, $aSecondaryDomains)) {
            print("$Secondary not found\n");
            return 0;
        }
    }
    foreach my $Secondary (@$aSecondaryDomains) {
        unless(ElementOf($Secondary, @$SECONDARYDOMAINS)) {
            print("Extra domain $Secondary found\n");
            return 0;
        }
    }

    return 1;
}

sub test {
    _TestRedirect();

    local $SOAP::Transport::HTTP::Client::USERAGENT_CLASS = 'DE_EPAGES::WebInterface::API::FakeRequest';

    ok(_TestShopService($PRIMARYDOMAIN), "ShopService on $PRIMARYDOMAIN");
    foreach my $Secondary (@$SECONDARYDOMAINS) {
        ok(_TestShopService($Secondary), "ShopService on $Secondary");
    }
    _TestUmlautSecondary();
}

__PACKAGE__->runScript;
